﻿using Rhino;
using Rhino.DocObjects;
using Rhino.Geometry;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;

namespace Test
{
    internal class PluginHandler
    {
        private static PluginHandler g_instance;
        private TestPlugIn _plugin;

        public static PluginHandler Instance => g_instance ?? (g_instance = new PluginHandler());

        private static uint g_doc_runtime_serial_number;
        public bool IsEnabled { get; private set; }

        private Dictionary<Guid, bool> _isDirty = new Dictionary<Guid, bool>();

        public void Enable(bool enable, TestPlugIn plugin)
        {
            _plugin = plugin;
            IsEnabled = enable;
            if (enable)
            {
                RhinoDoc.AddRhinoObject += RhinoDoc_AddRhinoObject;
            }
        }

        private void RhinoDoc_AddRhinoObject(object sender, RhinoObjectEventArgs e)
        {
            _isDirty[e.ObjectId] = true;
            HookIdle(e.TheObject.Document.RuntimeSerialNumber);
        }

        public void HookIdle(uint docRuntimeSerialNumber)
        {
            g_doc_runtime_serial_number = docRuntimeSerialNumber;
            RhinoApp.Idle += RhinoApp_Idle;
        }

        private void UnhookIdle()
        {
            g_doc_runtime_serial_number = 0;
            RhinoApp.Idle -= RhinoApp_Idle;
        }

        private void RhinoApp_Idle(object sender, EventArgs e)
        {
            var doc = RhinoDoc.FromRuntimeSerialNumber(g_doc_runtime_serial_number);
            if (doc != null)
            {
                OnChange(doc);
            }
            UnhookIdle();
        }

        public void SetDirty(Guid objectId)
        {
            _isDirty[objectId] = true;
        }

        public void OnChange(RhinoDoc rhinoDoc)
        {
            var objectsToRecalculate = rhinoDoc.Objects.Where(o => _isDirty.ContainsKey(o.Id) && _isDirty[o.Id] == true);

            foreach (var rhinoObject in objectsToRecalculate)
            {
                _isDirty[rhinoObject.Id] = false;
                if(rhinoObject.ObjectType == ObjectType.Curve)
                {
                    var areaProperties = AreaMassProperties.Compute((Curve)rhinoObject.Geometry);
                    Thread.Sleep(250);
                    var testBool = rhinoObject.GetTestBool();
                    _plugin.ObjectAreas[rhinoObject.Id] = testBool ? areaProperties.Area : areaProperties.Area * 0.5;
                }
            }

            rhinoDoc.Views.Redraw();
            _plugin.OnRecalculated();
        }
    }
}
