﻿using Test.Properties;
using Rhino.DocObjects;
using Rhino.UI;
using System.Drawing;

namespace Test.UI.ObjectSettings
{
    public class TestObjectProperties : ObjectPropertiesPage
    {
        public TestPlugIn Plugin;
        public RhinoObject RhinoObject;

        public TestObjectProperties(TestPlugIn plugin)
        {
            Plugin = plugin;
            CollapsibleSectionHolder = new ObjectSettingsPanel(this);
        }

        public override string EnglishPageTitle => "Test";


        public override Icon PageIcon(Size size)
        {
            return new Icon(Resources.icon_smile, size);
        }

        public override void UpdatePage(ObjectPropertiesPageEventArgs e)
        {
            if (e.ObjectCount == 1)
            {
                var newObject = e.Objects[0];

                if (RhinoObject == null || RhinoObject.Disposed || RhinoObject.Id != newObject.Id)
                {
                    RhinoObject = newObject;
                    CollapsibleSectionHolder.ResetSections();
                }
            }
            base.UpdatePage(e);
        }

        public override PropertyPageType PageType => PropertyPageType.Custom;

        public override bool ShouldDisplay(ObjectPropertiesPageEventArgs e)
        {
            if (e.ObjectCount != 1) return false;
            return Plugin.ObjectAreas.ContainsKey(e.Objects[0].Id);
        }

        public override ObjectType SupportedTypes => ObjectType.AnyObject;

        public sealed override object PageControl => CollapsibleSectionHolder;

        private ObjectSettingsPanel CollapsibleSectionHolder { get; }

        public void CalculationsUpdated()
        {
            CollapsibleSectionHolder.UpdateSections();
        }
    }
}
